package HomebrewCryptor;

use Term::ReadKey;
use Crypt::CBC;

my $PRINT_INFORMATION = 0;
my $PRINT_ERROR = 1;

my $MODULE_NAME = 'HomebrewCryptor';
my $PRINT_INFORMATION_PREFIX = 'INFORMATION - ' . $MODULE_NAME . ' - ';
my $PRINT_ERROR_PREFIX = 'ERROR - ' . $MODULE_NAME . ' - ';
my $PRINT_LOCAL_PREFIX = '';



my $DEFAULT_CIPHER = 'Blowfish';
my $DEFAULT_PASSWORD_PROMPT = 'Enter password: ';



my $CIPHER_KEY = 'cipher';
my $PASSWORD_PROMPT_KEY = 'prompt';
my $PASSWORD_KEY = 'password';



my $TRUE = 1;
my $FALSE = 0;

my $NORMAL_RETURN_STATUS = $TRUE;
my $ERROR_RETURN_STATUS = $FALSE;



my $READ_ONLY_OPEN_MODE = '<';
my $WRITE_CLOBBER_OPEN_MODE = '+>';

my $SILENT_READ_MODE = 2;
my $NORMAL_READ_MODE = 0;
my $BLOCKING_READ_KEY = 0;



my $NEWLINE_CHARACTER = "\n";

my $MAX_READ_LENGTH = 2 ** 20;



# This is a temporary return status holder
my $returnStatus;



sub new
{
    my $PRINT_LOCAL_PREFIX = 'new() - ';
   
    my $class = shift;
    my $this = { $CIPHER_KEY => $DEFAULT_CIPHER,
		 $PASSWORD_PROMPT_KEY => $DEFAULT_PASSWORD_PROMPT,
		 @_ };
    bless($this, $class);

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Creating new HomebrewCryptor.\n");
    }
    
    return $this;
}



sub encryptData
{
    my $PRINT_LOCAL_PREFIX = 'encryptData() - ';

    my $this = shift;
    my ($plainText) = @_;

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Encrypting data.\n");
    }

    my $cryptor = $this->createNewCryptor();
    return ($cryptor->encrypt($plainText));
}



sub decryptData
{
    my $PRINT_LOCAL_PREFIX = 'decryptData() - ';

    my $this = shift;
    my ($cipherText) = @_;

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Decrypting data.\n");
    }

    my $cryptor = $this->createNewCryptor();
    return ($cryptor->decrypt($cipherText));
}

sub encryptDataToFile
{
    my $PRINT_LOCAL_PREFIX = 'encryptDataToFile() - ';

    my $this = shift;
    my ($plainText, $outputFilePath) = @_;

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Encrypting data to file.\n");
    }

    my $cipherText = $this->encryptData($plainText);
    $returnStatus = $this->writeFile($cipherText, $outputFilePath);

    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem writing cipherText to file.\n");
	}

	return $ERROR_RETURN_STATUS
    }

    return $NORMAL_RETURN_STATUS;
}



sub decryptDataFromFile
{
    my $PRINT_LOCAL_PREFIX = 'decryptDataFromFile() - ';

    my $this = shift;
    my ($inputFilePath) = @_;

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Decrypting data from file.\n");
    }
    
    $returnStatus = $this->readFile($inputFilePath);

    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem reading cipherText from file.\n");
	}

	return $ERROR_RETURN_STATUS;
    }

    my $cipherText = $returnStatus;

    return ($this->decryptData($cipherText));
}



sub encryptFileToFile
{
    my $PRINT_LOCAL_PREFIX = 'encryptFileToFile() - ';

    my $this = shift;
    my ($inputFilePath, $outputFilePath) = @_;

    $returnStatus = $this->readFile($inputFilePath);

    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem reading plainText from file.\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    my $plainText = $returnStatus;

    $returnStatus = $this->encryptDataToFile($plainText, $outputFilePath);

    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem encrypting to file.\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    return $NORMAL_RETURN_STATUS;
}



sub getPassword
{
    my $PRINT_LOCAL_PREFIX = 'getPassword() - ';

    my $this = shift;

    if (!($this->{$PASSWORD_KEY}))
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Getting password\n");
	}

	my $newPassword = '';
	my $currentKey = '';

	print($this->{$PASSWORD_PROMPT_KEY});
	ReadMode($SILENT_READ_MODE);

	while ($currentKey ne $NEWLINE_CHARACTER)
	{
	    $newPassword = $newPassword . $currentKey;
	    $currentKey = ReadKey($BLOCKING_READ_KEY);
	}
	$this->{$PASSWORD_KEY} = $newPassword;

	print($NEWLINE_CHARACTER);

	ReadMode($NORMAL_READ_MODE);
    }
    else
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Reusing cached password\n");
	}
    }

    return $this->{$PASSWORD_KEY};
}



sub createNewCryptor
{
    my $CRYPTOR_KEY_KEY = 'key';
    my $CRYPTOR_CIPHER_KEY = 'cipher';

    my $this = shift;

    my $cryptor = Crypt::CBC->new({ $CRYPTOR_KEY_KEY => $this->getPassword(), 
				    $CRYPTOR_CIPHER_KEY => $this->{$CIPHER_KEY} });

    return $cryptor;
}

sub readFile
{
    my $PRINT_LOCAL_PREFIX = 'readFile() - ';

    my $this = shift;
    my ($inputFilePath) = @_;

    $returnStatus = open(INPUT_FILE, $READ_ONLY_OPEN_MODE, $inputFilePath);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem opening file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    my $inputData;

    $returnStatus = read(INPUT_FILE, $inputData, $MAX_READ_LENGTH);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem reading file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    $returnStatus = close(INPUT_FILE);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem reading file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    return $inputData;
}

sub writeFile
{
    my $PRINT_LOCAL_PREFIX = 'writeFile() - ';

    my $this = shift;
    my ($outputData, $outputFilePath) = @_;

    $returnStatus = open(OUTPUT_FILE, $WRITE_CLOBBER_OPEN_MODE, $outputFilePath);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem opening file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    $returnStatus = print(OUTPUT_FILE $outputData);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem writing file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    $returnStatus = close(OUTPUT_FILE);
    if (!($returnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Problem closing file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    return $NORMAL_RETURN_STATUS;
}


return $TRUE;
