/*
 * $Id: HTMLPostUtils.java,v 1.4.2.1 2002/09/17 01:52:21 jsergent Exp $
 * HTMLPostUtils.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import java.net.URLDecoder;
import java.util.StringTokenizer;
import org.w3c.dom.Document;

/**
 * Utility functions for implementing the HTML Form Post profile.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.1 $
 */
public class HTMLPostUtils {
    /**
     * Decodes an x-www-form-urlencoded string resulting from an incoming
     * HTML Form Post profile message.  Returns an XML Document with the
     * parsed XML contained in the message.
     *
     * @param query The query string.
     * @param param The name of the query parameter to look for.
     * @return The parsed XML document.
     * @exception Exception Thrown if URL decoding or XML parsing fails
     */
    public static Document decode(String query, String param) throws Exception {
	StringTokenizer t = new StringTokenizer(query, "&=", true);
	while (t.hasMoreTokens()) {
	    String x = t.nextToken();
	    if (x.equals(param)) {
		t.nextToken();
		String b64msg = URLDecoder.decode(t.nextToken());
		if (Config.debug) {
		    System.err.println("x:" + x + " param:" + param
				       + " b64msg:" + b64msg);
		}
		byte[] bytemsg = URLUtils.base64Decode(b64msg);
		String msg = new String(bytemsg);
		if (Config.debug) {
		    System.err.println("XMLUtils parse msg:" +  msg);
		}
		return XMLUtils.parse(msg);
	    }
	}
	return null;
    }

    /**
     * Encode text using HTML entities for markup-significant characters
     * so that the browser decodes it properly.  Necessary if you want to do
     * &lt;input type="hidden" value="hello, "bob""&gt; &lt;!-- WRONG! --&gt;
     * which should be
     * &lt;input type="hidden" value="hello, &amp;quot;bob&amp;quot;"&gt;
     * so we encode " in addition to the usual cases of <, >, and &.
     */
    public static String htmlEscape(String in) {
	StringBuffer out = new StringBuffer();
	for (int i = 0; i < in.length(); i++) {
	    char c = in.charAt(i);
	    switch (c) {
		case '"':	out.append("&quot;"); break;
		case '&':	out.append("&amp;"); break;
		case '<':	out.append("&lt;"); break;
		case '>':	out.append("&gt;"); break;
		default:	out.append(c); break;
	    }
	}
	return out.toString();
    }

    /**
     * Encodes an XML message for HTML Form Post profile transmission.
     *
     * @param param The form field name to use
     * @param xml The XML to transmit
     * @param destURL The URL that the form should be posted to
     * @return HTML fragment containing a form with a hidden field and
     *         a submit button.
     */
    public static String encode(String param, String xml, String destURL) throws Exception {
	StringBuffer html = new StringBuffer();
        html.append("<form action=\"");
	html.append(destURL);
	html.append("\" method=\"POST\">");
	html.append("<input type=\"hidden\" name=\"");
	html.append(htmlEscape(param));
	html.append("\" value=\"");
	html.append(htmlEscape(URLUtils.base64Encode(xml.getBytes())));
	html.append("\"/><input type=\"submit\"/></form>");
	return html.toString();
    }
}
