/*
 * $Id: LibertyProfile.java,v 1.4.2.1 2002/09/17 01:52:24 jsergent Exp $
 * LibertyLogoutNotification.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;

import java.util.Map;

/**
 * Implements enumeration of Liberty profile identifiers.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.1 $
 */
public class LibertyProfile implements Comparable {
    private final String name;
    private static int nextOrdinal = 0;
    private static Map map = new java.util.HashMap();
    private final int ordinal = nextOrdinal++;

    /** 
     * Constructor, to be used with static values
     * @param name the profile URI
     */
    protected LibertyProfile(String name) {
	this.name = name;
	map.put(name, this);
    }
    /**
     * Returns the profile URI of the profile
     * @return the profile URI
     */
    public String toString() { return name; }
    public int compareTo(Object o) {
	return ordinal - ((LibertyProfile) o).ordinal;
    }

    /** Browser artifact single sign-on profile */
    public static final LibertyProfile BRWS_ART
	= new LibertyProfile(LibertyElement.BRWS_ART_PROFILE_URI);
    /** Browser post single sign-on profile */
    public static final LibertyProfile BRWS_POST
	= new LibertyProfile(LibertyElement.BRWS_POST_PROFILE_URI);
    /** WML post single sign-on profile */
    public static final LibertyProfile WML_POST
	= new LibertyProfile(LibertyElement.WML_POST_PROFILE_URI);
    /** Liberty-enabled client and proxy single sign-on profile */
    public static final LibertyProfile LECP
	= new LibertyProfile(LibertyElement.LECP_PROFILE_URI);
    /** IDP-initiated HTTP federation termination profile */
    public static final LibertyProfile FT_IDP_HTTP
        = new LibertyProfile(LibertyElement.FT_IDP_HTTP_PROFILE_URI);
    /** IDP-initiated SOAP federation termination profile */
    public static final LibertyProfile FT_IDP_SOAP
        = new LibertyProfile(LibertyElement.FT_IDP_SOAP_PROFILE_URI);
    /** SP-initiated HTTP federation termination profile */
    public static final LibertyProfile FT_SP_HTTP
        = new LibertyProfile(LibertyElement.FT_SP_HTTP_PROFILE_URI);
    /** SP-initiated SOAP federation termination profile */
    public static final LibertyProfile FT_SP_SOAP
        = new LibertyProfile(LibertyElement.FT_SP_SOAP_PROFILE_URI);
    /** IDP-initiated HTTP single logout profile */
    public static final LibertyProfile SL_IDP_HTTP
        = new LibertyProfile(LibertyElement.SL_IDP_HTTP_PROFILE_URI);
    /** IDP-initiated HTTP GET single logout profile */
    public static final LibertyProfile SL_IDP_HTTP_GET
        = new LibertyProfile(LibertyElement.SL_IDP_HTTP_GET_PROFILE_URI);
    /** IDP-initiated SOAP single logout profile */
    public static final LibertyProfile SL_IDP_SOAP
        = new LibertyProfile(LibertyElement.SL_IDP_SOAP_PROFILE_URI);
    /** SP-initiated HTTP single logout profile */
    public static final LibertyProfile SL_SP_HTTP
        = new LibertyProfile(LibertyElement.SL_SP_HTTP_PROFILE_URI);
    /** SP-initiated SOAP single logout profile */
    public static final LibertyProfile SL_SP_SOAP
        = new LibertyProfile(LibertyElement.SL_SP_SOAP_PROFILE_URI);
    

    /**
     * Looks up singleton instance associated with a profile URI.
     * For example, valueOf(LibertyElement.BRWS_ART_PROFILE_URI) returns
     * LibertyProfile.BRWS_ART.
     *
     * @param name The URI
     * @return The singleton instance associated with the URI
     */
    public static LibertyProfile valueOf(String name) {
	return (LibertyProfile) map.get(name);
    }
}
