/*
 * $Id: RequestProcessorAction.java,v 1.3.2.1 2002/09/17 01:52:26 jsergent Exp $
 * RequestProcessorAction.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */
package com.sun.liberty;

/**
 * Implements enumeration of request processor action identifiers.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.3.2.1 $
 */
public class RequestProcessorAction implements Comparable {
    private final String name;
    private static int nextOrdinal = 0;
    private final int ordinal = nextOrdinal++;

    private RequestProcessorAction(String name) { this.name = name; }
    public String toString() { return name; }
    public int compareTo(Object o) {
	return ordinal - ((RequestProcessorAction) o).ordinal;
    }

    /** Request was processed, return normal response to user. */
    public static final RequestProcessorAction RETURN_RESPONSE
	= new RequestProcessorAction("RETURN_RESPONSE");
    /** Attempt to authenticate user, then retry request. */
    public static final RequestProcessorAction ATTEMPT_USER_AUTH
	= new RequestProcessorAction("ATTEMPT_USER_AUTH");
    /** Obtain user consent, then retry request. */
    public static final RequestProcessorAction OBTAIN_USER_CONSENT
	= new RequestProcessorAction("OBTAIN_USER_CONSENT");
    /** Request processing failed.  Return failure and do not try again. */
    public static final RequestProcessorAction FAILED
	= new RequestProcessorAction("FAILED");
}
