/*
 * $Id: SAMLConditions.java,v 1.5.2.1 2002/09/17 01:52:30 jsergent Exp $
 * SAMLConditions.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:Conditions element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */

public class SAMLConditions extends SAMLElement {
    Date notBefore = null;
    Date notOnOrAfter = null;
    List conditions = null;

    public void parse(Element e) {
	setNotBefore(parseDate(e.getAttributeNS(null, "NotBefore")));
	setNotOnOrAfter(parseDate(e.getAttributeNS(null, "NotOnOrAfter")));
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    if (isElement(child, SAML_NS_URI, "AudienceRestrictionCondition")) {
		SAMLAudienceRestrictionCondition c = new SAMLAudienceRestrictionCondition();
		c.parse(child);
		List set = getConditions();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setConditions(set);
	    }
	}
    }
	
    
    public Element getElement(Document doc) {
        Element c = doc.createElementNS(SAML_NS_URI, SAML_NS_ID + ":Conditions");
        addAttributesTo(c);
        addChildrenTo(c);
        return c;
    }

    protected void addAttributesTo(Element c) {
        if (notBefore != null) {
            c.setAttributeNS(null, "NotBefore", formatDate(notBefore));
        }
        if (notOnOrAfter != null) {
            c.setAttributeNS(null, "NotOnOrAfter", formatDate(notOnOrAfter));
        }
    }

    protected void addChildrenTo(Element c) {
        if (conditions != null) {
            Iterator i = conditions.iterator();
            while (i.hasNext()) {
                SAMLCondition cn = (SAMLCondition) i.next();
                c.appendChild(cn.getElement(c.getOwnerDocument()));
            }
        }
    }

    
    public Date getNotBefore() {
        return notBefore;
    }

    public void setNotBefore(Date value) {
        notBefore = value;
    }

    public Date getNotOnOrAfter() {
        return notOnOrAfter;
    }

    public void setNotOnOrAfter(Date value) {
        notOnOrAfter = value;
    }

    public List getConditions() {
        return conditions;
    }

    public void setConditions(List value) {
        conditions = value;
    }
}
