/*
 * $Id: SAMLPAbstractRequest.java,v 1.9.2.2 2002/09/17 01:52:31 jsergent Exp $
 * SAMLPAbstractRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import com.sun.liberty.XMLUtils;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 * Data bindings for the samlp:AbstractRequestType type.  Get and set
 * methods correspond to child elements and attributes in the XML schema
 * for this element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.9.2.2 $
 */

public abstract class SAMLPAbstractRequest extends SAMLElement {
    private String requestID = generateIDType();
    private int majorVersion = 1;
    private int minorVersion = 0;
    private Date issueInstant = new Date();
    private List respondWith = null;
    private String signedBy = null;
    private String receivedFrom = null;

    public void parse(Element e) {
	setRequestID(e.getAttributeNS(null, "RequestID"));
	setMajorVersion(Integer.valueOf(e.getAttributeNS(null, "MajorVersion"))
		.intValue());
	setMinorVersion(Integer.valueOf(e.getAttributeNS(null, "MinorVersion"))
		.intValue());
	setIssueInstant(parseDate(e.getAttributeNS(null, "IssueInstant")));

	Element signature = null;

	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAMLP_NS_URI, "RespondWith")) {
		if (respondWith == null) {
		    respondWith = new LinkedList();
		}
		respondWith.add(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, DSIG_NS_URI, "Signature")) {
		signature = child;
	    }
	}

        if (signature != null) {
            setSignedBy(XMLUtils.verify(signature, receivedFrom) ? receivedFrom : null);
        }
    }

    protected void addAttributesTo(Element s) {
	s.setAttributeNS(null, "RequestID", requestID);
	s.setAttributeNS(null, "MajorVersion", new Integer(majorVersion).toString());
	s.setAttributeNS(null, "MinorVersion", new Integer(minorVersion).toString());
	s.setAttributeNS(null, "IssueInstant", formatDate(issueInstant));
    }

    protected void addChildrenTo(Element s) {
	if (respondWith != null) {
	    Document doc = s.getOwnerDocument();
	    Iterator i = respondWith.iterator();
	    while (i.hasNext()) {
		String rw = (String) i.next();
		Element rwe = doc.createElementNS(SAMLP_NS_URI,
			SAMLP_NS_ID + ":RespondWith");
		s.appendChild(rwe);
		rwe.setAttributeNS(XMLNS_NS_URI, "xmlns:lib", com.sun.liberty.LibertyElement.LIB_NS_URI);
		rwe.setAttributeNS(XMLNS_NS_URI, "xmlns:samlp", SAMLP_NS_URI);
		rwe.appendChild(doc.createTextNode(rw));
	    }
	}
	if (signedBy != null) {
	    XMLUtils.sign(s);
	}
    }

    public String getRequestID() {
	return requestID;
    }

    public void setRequestID(String value) {
	requestID = value;
    }

    public int getMajorVersion() {
	return majorVersion;
    }

    public void setMajorVersion(int value) {
	majorVersion = value;
    }

    public int getMinorVersion() {
	return minorVersion;
    }

    public void setMinorVersion(int value) {
	minorVersion = value;
    }

    public Date getIssueInstant() {
	return issueInstant;
    }

    public void setIssueInstant(Date value) {
	issueInstant = value;
    }

    public List getRespondWith() {
	return respondWith;
    }

    public void setRespondWith(List value) {
	respondWith = value;
    }

    public String getSignedBy() {
	return signedBy;
    }

    public void setSignedBy(String value) {
	signedBy = value;
    }

    public void setReceivedFrom(String value) {
	receivedFrom = value;
    }
}
