/*
 * $Id: FederationTerminationServlet.java,v 1.1.2.2 2002/09/17 01:52:34 jsergent Exp $
 * FederationTerminationServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.LibertySoapFederationTerminationNotification;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.SPDescriptor;
import com.sun.liberty.SoapUtils;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 *
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.1.2.2 $
 */

public class FederationTerminationServlet extends HttpServlet {
    static ProviderDescriptorFactory dir;

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }

    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
	LibertyUserSession lus = getLibertySession(req);
	String nonce = req.getParameter("nonce");

	if (nonce == null || !(nonce.equals(lus.getFCNonce()))) {
	    resp.sendRedirect(Config.federationPageForRedirect);
	    return;
	}

	lus.newFCNonce();
	LibertyUser lu = lus.getUser();
	if (lu == null) {
	    throw new ServletException("Current user is null");
	}
	String id = req.getParameter("_liberty_id");
	ProviderDescriptor prov = dir.find(id);
	if (prov == null) {
	    throw new ServletException("Cannot find IDP");
	}
	LibertySoapFederationTerminationNotification n;
	n = new LibertySoapFederationTerminationNotification();
	n.setProviderID(Config.localProviderID);
	n.setSignedBy(Config.localProviderID);
	try {
	    if (prov instanceof SPDescriptor) {
		n.setNameIdentifier(lu.getLocalNameFor((SPDescriptor) prov));
		SoapUtils.send(n, prov.getSoapEndpoint().toString());
		lu.setFederationStatus((SPDescriptor) prov, false);
		resp.sendRedirect(Config.federationPageForRedirect);
		return;
	    } else if (prov instanceof IDPDescriptor) {
		n.setNameIdentifier(lu.getLocalNameFor((IDPDescriptor) prov));
		SoapUtils.send(n, prov.getSoapEndpoint().toString());
		lu.setFederationStatus((IDPDescriptor) prov, false);
		resp.sendRedirect(Config.federationPageForRedirect);
		return;
	    } 
	} catch (Exception e) {
	    throw new ServletException("Federation termination failed", e);
	}
	throw new ServletException("Provider is neither SP nor IDP");
    }
}
