/*
 * $Id: SoapServlet.java,v 1.7.2.1 2002/09/17 01:52:35 jsergent Exp $
 * SoapServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty.servlets;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;

import com.sun.liberty.RequestProcessorAction;
import com.sun.liberty.FederationTerminationNotificationProcessor;
import com.sun.liberty.LogoutNotificationProcessor;
import com.sun.liberty.LibertyElement;
import com.sun.liberty.LibertySoapFederationTerminationNotification;
import com.sun.liberty.LibertySoapLogoutNotification;

import com.sun.liberty.SAMLRequestProcessor;
import com.sun.liberty.SimpleSAMLRequestProcessor;
import com.sun.liberty.saml.SAMLElement;
import com.sun.liberty.saml.SAMLPSoapRequest;
import com.sun.liberty.saml.SAMLPSoapResponse;

import ucb.eecs.sahara.sso.anp.MessageANPRequestProcessor;
import ucb.eecs.sahara.sso.anp.SimpleMessageANPRequestProcessor;
import ucb.eecs.sahara.sso.anp.ANPRequestProcessorAction;
import ucb.eecs.sahara.sso.anp.ANPElement;
import ucb.eecs.sahara.sso.anp.ANPSoapRequest;
import ucb.eecs.sahara.sso.anp.ANPSoapResponse;

import javax.xml.messaging.JAXMServlet;
import javax.xml.messaging.ReqRespListener;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.MimeHeaders;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletException;


/**
 * JAXM servlet to dispatch incoming requests to appropriate request
 * processors
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.7.2.1 $
 */

public class SoapServlet extends JAXMServlet implements ReqRespListener {
    private boolean isElement(Name n, String ns, String name) {
	String nameNS = n.getURI();
	String nameName = n.getLocalName();
	return (ns.equals(nameNS) && (name.compareToIgnoreCase(nameName) == 0));
    }

    public void doPost(HttpServletRequest httpReq, HttpServletResponse httpResp)
                         throws ServletException {
        /*
         * We need HttpServletRequest and HttpServletResponse info if Liberty
         * authentication is requested via the Authentication Negotiation
         * Protocol
         * XXX: Should also check for TLS and refuse name and password
         * in that case too.
         */
        DebugUtils.debugTime("SoapServlet", "Entering doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);

	try {
	    /* Get the SOAP message in the request */
	    ServletInputStream reqInputStream = httpReq.getInputStream();
	    MimeHeaders reqHeaders = getHeaders(httpReq);
	    SOAPMessage soapReq = 
		msgFactory.createMessage(reqHeaders, reqInputStream);

	    /* Process the SOAP message and, if a SOAP message must be sent 
	     * in response, send it.
             */ 
	    SOAPMessage soapResp = null;
	    soapResp = this.onMessage(httpReq, httpResp, soapReq);
	    if (soapResp != null) {
		if (soapResp.saveRequired()) {
		    soapResp.saveChanges();
		}

	        SOAPBody respBody = 
		    soapResp.getSOAPPart().getEnvelope().getBody();
		if (respBody.hasFault()) {
		    httpResp.setStatus(500);
		} else {
		    httpResp.setStatus(200);
		}

		putHeaders(soapResp.getMimeHeaders(), httpResp);
		ServletOutputStream respOutputStream = httpResp.getOutputStream();
		soapResp.writeTo(respOutputStream);
		respOutputStream.flush();
	    }
	} catch (Exception e) {
		e.printStackTrace();
	}

        DebugUtils.debugTime("SoapServlet", "Leaving doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }

    public SOAPMessage onMessage(SOAPMessage msg) {
	return onMessage(null, null, msg);
    }

    public SOAPMessage onMessage(HttpServletRequest httpReq,
	                        HttpServletResponse httpResp, SOAPMessage msg) {
        DebugUtils.debugTime("SoapServlet", "Entering onMessage",
                             DebugUtils.LOW_LEVEL_FUNC);

	try {
	    if (Config.debug) {
		System.err.println("SoapServlet got message:");
		try { msg.writeTo(System.err); } catch (Exception e) {}
	    }
	    /*
	     * Looks at the name of the first element in the body of the
	     * message to figure out what to do with it.
	     */
	    SOAPBody body = msg.getSOAPPart().getEnvelope().getBody();
	    SOAPElement bodyElt = null;
	    java.util.Iterator i = body.getChildElements();
	    while (i.hasNext() && bodyElt == null) {
		Object nextObj = i.next();
		if (nextObj instanceof SOAPElement) {
		    bodyElt = (SOAPElement) nextObj;
		}
	    }
	    /* JAXM Name object for first element in SOAP body: */
	    Name eltName = bodyElt.getElementName();

	    SOAPMessage reply = msgFactory.createMessage();
	    SOAPBody replyBody = reply.getSOAPPart().getEnvelope().getBody();

	    try {
		if (isElement(eltName, SAMLElement.SAMLP_NS_URI, "Request")) {
		    SAMLPSoapRequest r = new SAMLPSoapRequest();
		    r.parse(msg);
                    DebugUtils.debugTime("SoapServlet", "SAML Request parsed",
                                         DebugUtils.LOW_LEVEL_FUNC);
		    if (Config.debug) {
			System.err.println("msg is now:");
			try { msg.writeTo(System.err); } catch (Exception e) {}
		    }

		    SAMLRequestProcessor rp = new SimpleSAMLRequestProcessor();
		    SAMLPSoapResponse response = new SAMLPSoapResponse();
		    RequestProcessorAction action;
		    action = rp.process(r, (SAMLPSoapResponse) response);
                    DebugUtils.debugTime("SoapServlet", "SAML Request processed",
                                         DebugUtils.LOW_LEVEL_FUNC);
		    if (action == RequestProcessorAction.RETURN_RESPONSE) {
			response.addToBody(reply);
                        DebugUtils.debugTime("SoapServlet", "SAML Reply built",
                                             DebugUtils.LOW_LEVEL_FUNC);
			if (Config.debug) {
			    System.err.println("SoapServlet: reply:");
			    reply.writeTo(System.err);
			}
			return reply;
		    } 
		} else if (isElement(eltName, LibertyElement.LIB_NS_URI,
			             "FederationTerminationNotification")) {
		    LibertySoapFederationTerminationNotification n;
		    n = new LibertySoapFederationTerminationNotification();
		    n.parse(msg);
                    DebugUtils.debugTime("SoapServlet",
                                         "Liberty FederationTerminationNotification parsed",
                                         DebugUtils.LOW_LEVEL_FUNC);

		    FederationTerminationNotificationProcessor np;
		    np = new FederationTerminationNotificationProcessor();
		    RequestProcessorAction action = np.process(n, null);
                    DebugUtils.debugTime("SoapServlet",
                                         "Liberty FederationTerminationNotification processed",
                                         DebugUtils.LOW_LEVEL_FUNC);
		    if (action == RequestProcessorAction.RETURN_RESPONSE) {
			// XXX: We want to be a one way listener in this
			// case, I think.... should not return anything.
			// Check JAXM docs to see what is possible.  May
			// have to be broken on this for now.
			return reply;
		    }
		} else if (isElement(eltName, LibertyElement.LIB_NS_URI,
                                     "LogoutNotification")) {
		    if (Config.debug) {
			System.err.println("Got SOAP logout notification");
		    }
                    LibertySoapLogoutNotification n;
                    n = new LibertySoapLogoutNotification();
                    n.parse(msg);
                    DebugUtils.debugTime("SoapServlet",
                                         "Liberty LogoutNotification parsed",
                                         DebugUtils.LOW_LEVEL_FUNC);

                    LogoutNotificationProcessor np;
                    np = new LogoutNotificationProcessor();
                    RequestProcessorAction action = np.process(n);
                    DebugUtils.debugTime("SoapServlet",
                                         "Liberty LogoutNotification processed",
                                         DebugUtils.LOW_LEVEL_FUNC);
                    if (action == RequestProcessorAction.RETURN_RESPONSE) {
                        return reply;
                    }
                } else if (isElement(eltName, ANPElement.ANP_NS_URI,
				     "Request")) {
                    ANPSoapRequest r = new ANPSoapRequest();
                    r.parse(msg);
                    DebugUtils.debugTime("SoapServlet",
                                         "ANP Request parsed",
                                         DebugUtils.LOW_LEVEL_FUNC);
		    if (Config.debug) {
                        System.err.println("msg is now:");
                        try { msg.writeTo(System.err); } catch (Exception e) {}
                    }

                    MessageANPRequestProcessor rp = new SimpleMessageANPRequestProcessor();
                    ANPSoapResponse resp = new ANPSoapResponse();
                    ANPRequestProcessorAction action;
                    action = rp.process(httpReq, httpResp, r, resp);
                    DebugUtils.debugTime("SoapServlet",
                                         "ANP Request processed",
                                         DebugUtils.LOW_LEVEL_FUNC);
                    if (action == ANPRequestProcessorAction.RETURN_RESPONSE) {
                        resp.addToBody(reply);
                        DebugUtils.debugTime("SoapServlet",
                                             "ANP Reply built",
                                             DebugUtils.LOW_LEVEL_FUNC);
                        if (Config.debug) {
                            System.err.println("SoapServlet: reply:");
                            reply.writeTo(System.err);
                        }
                        return reply;
		    }
                }
	    } catch (Exception e) {
		e.printStackTrace();
	    }

	    if (Config.debug) { System.err.println("Server SOAP fault"); }
	    SOAPFault fault = replyBody.addFault();
	    fault.setFaultCode("SOAP-ENV:Server");
	    fault.setFaultString("Message processing failed!");

	    return reply;
	} catch (Throwable e) {
	    e.printStackTrace();
	    throw new RuntimeException("SOAP error... oops.");
	}
    }
}
