package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.anp.ANPElement;
import com.sun.liberty.saml.SAMLSubjectStatement;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for anp:AuthnCapabilitiesStatement element.  Get and set
 * methods correspond to child elements and attributes in the XML schema
 * for this element.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class ANPAuthnCapabilitiesStatement extends SAMLSubjectStatement implements ANPElement {
    private String nonce = null;
    private Date lastUpdateInstant = null;
    private List idpGroups = null;
    private List authnMethods = null;
    private List chargingOptions = null;
    private List services = null;


    public void parse(Element e) {
	super.parse(e);
        setLastUpdateInstant(parseDate(e.getAttributeNS(null, "LastUpdateInstant")));
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
            if (isElement(child, ANP_NS_URI, "Nonce")) {
                setNonce(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "IDPGroup")) {
		ANPIDPGroup c = new ANPIDPGroup();
		c.parse(child);
		List set = getIDPGroups();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setIDPGroups(set);
	    } else if (isElement(child, ANP_NS_URI, "AuthnMethod")) {
		ANPAuthnMethod c = new ANPAuthnMethod();
		c.parse(child);
		List set = getAuthnMethods();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setAuthnMethods(set);
	    } else if (isElement(child, ANP_NS_URI, "ChargingOption")) {
		ANPChargingOption c = new ANPChargingOption();
		c.parse(child);
		List set = getChargingOptions();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setChargingOptions(set);
	    } else if (isElement(child, ANP_NS_URI, "Service")) {
		ANPService c = new ANPService();
		c.parse(child);
		List set = getServices();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setServices(set);
	    }
	}
    }

    public Element getElement(Document doc) {
	Element s;
	s = doc.createElementNS(ANP_NS_URI, ANP_NS_ID + ":AuthnCapabilitiesStatement");
	addAttributesTo(s);
	addChildrenTo(s);
	return s;
    }

    protected void addChildrenTo(Element s) {
	super.addChildrenTo(s);
        appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":Nonce", nonce);
	iterateAddChildren(s, idpGroups);
	iterateAddChildren(s, authnMethods);
	iterateAddChildren(s, chargingOptions);
	iterateAddChildren(s, services);
    }

    protected void addAttributesTo(Element s) {
        if (lastUpdateInstant != null) {
            s.setAttributeNS(null, "LastUpdateInstant",
                      formatDate(lastUpdateInstant));
        }
    }

    public String getNonce() {
        return nonce;
    }

    public void setNonce(String value) {
        nonce = value;
    }

    public Date getLastUpdateInstant() {
        return lastUpdateInstant;
    }

    public void setLastUpdateInstant(Date value) {
        lastUpdateInstant = value;
    }

    public List getIDPGroups() {
	return idpGroups;
    }

    public void setIDPGroups(List value) {
	idpGroups = value;
    }
    public List getAuthnMethods() {
	return authnMethods;
    }

    public void setAuthnMethods(List value) {
	authnMethods = value;
    }

    public List getChargingOptions() {
	return chargingOptions;
    }

    public void setChargingOptions(List value) {
	chargingOptions = value;
    }
 
    public List getServices() {
	return services;
    }

    public void setServices(List value) {
	services = value;
    }
}
