package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.XMLElement;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */
public class ANPChargingOption extends XMLElement implements ANPElement {
    private String chargingOptionID = null;
    private List chargingIntervals = null;
    private List userInfoDesignators = null;
    private List serviceIDReferences = null;

    private String chargingMode = null;
    private String timeUnit = null;
    private String unitPrice = null;
    private String fixPrice = null;

    public void parse(Element e) {
        NodeList list = e.getChildNodes();
        for (int j = 0; j < list.getLength(); j++) {
            if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
            Element child = (Element) list.item(j);
            if (isElement(child, ANP_NS_URI, "ChargingOptionID")) {
                setChargingOptionID(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "ChargingMode")) {
                setChargingMode(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "TimeUnit")) {
                setTimeUnit(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "UnitPrice")) {
                setUnitPrice(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "FixPrice")) {
                setFixPrice(child.getFirstChild().getNodeValue());
            //} else if (isElement(child, ANP_NS_URI, "ChargingInterval")) {
            //    ANPChargingInterval c = new ANPChargingInterval();
            //    c.parse(child);
            //    List set = getChargingIntervals();
            //    if (set == null) {
            //        set = new LinkedList();
            //    }
            //    set.add(c);
            //    setChargingIntervals(set);
            } else if (isElement(child, ANP_NS_URI, "UserInfoDesignator")) {
                ANPUserInfoDesignator c = new ANPUserInfoDesignator();
                c.parse(child);
                List set = getUserInfoDesignators();
                if (set == null) {
                    set = new LinkedList();
                }
                set.add(c);
                setUserInfoDesignators(set);
            } else if (isElement(child, ANP_NS_URI, "ServiceIDReference")) {
                List set = getServiceIDReferences();
                if (set == null) {
                    set = new LinkedList();
                }
                set.add(child.getFirstChild().getNodeValue());
                setServiceIDReferences(set);
            }
        }
    }

    public Element getElement(Document doc) {
	Element l = doc.createElementNS(ANP_NS_URI, "anp:ChargingOption");
	addChildrenTo(l);
	return l;
    }

    protected void addChildrenTo(Element s) {
	appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":ChargingOptionID", chargingOptionID);
	appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":ChargingMode", chargingMode);
	appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":TimeUnit", timeUnit);
	appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":UnitPrice", unitPrice);
	appendNewChildNSWithText(s, ANP_NS_URI,
            ANP_NS_ID + ":FixPrice", fixPrice);
        iterateAddChildren(s, chargingIntervals);
        iterateAddChildren(s, userInfoDesignators);
        iterateAddNewChildrenNSWithText(s, ANP_NS_URI, ANP_NS_ID + ":ServiceIDReference",
                                 serviceIDReferences);
    }

    public String getChargingOptionID() {
	return chargingOptionID;
    }

    public void setChargingOptionID(String value) {
	chargingOptionID = value;
    }

    public String getChargingMode() {
	return chargingMode;
    }

    public void setChargingMode(String value) {
	chargingMode = value;
    }

    public String getTimeUnit() {
	return timeUnit;
    }

    public void setTimeUnit(String value) {
	timeUnit = value;
    }

    public String getUnitPrice() {
	return unitPrice;
    }

    public void setUnitPrice(String value) {
	unitPrice = value;
    }

    public String getFixPrice() {
	return fixPrice;
    }

    public void setFixPrice(String value) {
	fixPrice = value;
    }

    public List getChargingIntervals() {
	return chargingIntervals;
    }

    public void setChargingIntervals(List value) {
	chargingIntervals = value;
    }

    public List getUserInfoDesignators() {
	return userInfoDesignators;
    }

    public void setUserInfoDesignators(List value) {
	userInfoDesignators = value;
    }

    public List getServiceIDReferences() {
	return serviceIDReferences;
    }

    public void setServiceIDReferences(List value) {
	serviceIDReferences = value;
    }
}
