package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.List;
import java.util.Iterator;

import ucb.eecs.sahara.sso.anp.saml.SAMLPQuery;
import ucb.eecs.sahara.sso.authn.SimpleL2Authenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;


/**
 * Processes compound authentication information contained in ANPAuthnQueries.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleANPAuthnQueryL2Processor implements ANPAuthnQueryL2Processor {

    private static final String macParam = "MacAddr";
    private static final String keyParam = "Key";

    public ANPRequestProcessorAction process(HttpServletRequest httpReq,
					     HttpServletResponse httpResp,
					     SAMLPQuery query,
                                             ANPAuthnQueryProcResult procResult)
    {
        try {
            String sessionKey = null;
            String macAddress = null;

            /* Extract authentication information from the ANP Authn/AuthnCont Query */
            List userInfo = null;
            if (query instanceof ANPAuthnQuery) {
                ANPAuthnQuery authnQuery = (ANPAuthnQuery) query;
                userInfo = authnQuery.getUserInfo();
            } else if (query instanceof ANPAuthnContQuery) {
                ANPAuthnContQuery authnContQuery = (ANPAuthnContQuery) query;
                userInfo = authnContQuery.getUserInfo();
            } else {
                if (Config.debug) {
                    System.err.println("Got invalid SAMLP query");
                }
                return ANPRequestProcessorAction.FAILED;
            }


            Iterator i = null;
            i = userInfo.iterator();
            while (i.hasNext()) {
                ANPUserInfo ui = (ANPUserInfo) i.next();

                if (ui.getAttributeName().equals(keyParam)) {
                    sessionKey = 
                        (String) ui.getAttributeValues().iterator().next();
                }
                if (ui.getAttributeName().equals(macParam)) {
                    macAddress = 
                        (String) ui.getAttributeValues().iterator().next();
                }
            }

            if (Config.debug) {
                System.err.println("MAC address: " + macAddress + 
				   ". Session key: " + sessionKey);
            }

            /* Check all required authentication information was present in the
             * ANPAuthnQuery */
            if ((sessionKey == null ) || (macAddress == null)) {
                if (Config.debug) {
                    System.err.println("Missing required user information for Compound authentication");
                }
                procResult.failureCode = "samlp:Sender";
                return ANPRequestProcessorAction.RETURN_RESPONSE;
            }

            /* Validate the authentication information */
            AuthenticatorResult authnResult = null;
            authnResult = SimpleL2Authenticator.authenticate(macAddress,
                                                             sessionKey);

            if (authnResult == AuthenticatorResult.AUTHN_SUCCESS) {
                if (Config.debug) {
                    System.err.println("Successful L2 info validation");
                }
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Successful";
            } else if (authnResult == AuthenticatorResult.AUTHN_FAILED) {
                if (Config.debug) {
                    System.err.println("Failed L2 info validation");
                }
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Failed";
            } else {
                if (Config.debug) {
                    System.err.println("Error at L2 info validation");
                }
                procResult.failureCode = "samlp:Receiver";
            }
            return ANPRequestProcessorAction.RETURN_RESPONSE;
        } catch (Exception e) {
            if (Config.debug) {
                System.err.println(e);
            }
            return ANPRequestProcessorAction.FAILED;
        }
    }
}
