package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.List;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.net.URL;

import ucb.eecs.sahara.sso.authn.LibertyAuthenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;
import ucb.eecs.sahara.sso.authn.LibertyAssertionControl;


/**
 * Processes ANPAuthnContQueries for LIBERTY.
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleLibertyANPAuthnContQueryProcessor implements ANPAuthnContQueryProcessor {

    public ANPRequestProcessorAction process(HttpServletRequest httpReq,
					     HttpServletResponse httpResp,
					     ANPAuthnContQuery authnContQuery,
                                             ANPAuthnQueryProcResult procResult)
	   throws Exception
    {
        try {
            String redirectedURL = null;
            String artifact = null;

            /* Extract redirect URL from the ANPAuthnContQuery */
            redirectedURL = authnContQuery.getRedirectedURL();
            if (Config.debug) {
                System.err.println("LIBERTY requested with redirectedURL " + redirectedURL);
            }

            /* Check all required user info was present in the ANPAuthnQuery */
            if (redirectedURL == null) {
                System.err.println("Missing required user information for LIBERTY authentication");
                procResult.failureCode = "samlp:Sender";
                return ANPRequestProcessorAction.RETURN_RESPONSE;
            }

            /* Extract Liberty artifact from the redirectedURL */
            URL url = new URL(redirectedURL);
            String urlQuery = url.getQuery();
            StringTokenizer urlQueryTokenizer = new StringTokenizer(urlQuery, "&");

            StringTokenizer parameterTokenizer = null;
            boolean cont = true;
            while (urlQueryTokenizer.hasMoreTokens() && cont) {
                parameterTokenizer = new StringTokenizer(urlQueryTokenizer.nextToken(), "=");
                if (parameterTokenizer.nextToken().equals("SAMLart")) {
                    artifact = java.net.URLDecoder.decode(parameterTokenizer.nextToken());
                    cont = false;
                }
            }

            if (artifact != null) {
                if (Config.debug) {
                    System.err.println("Received artifact: " + artifact);
                }
            }

            /* Extract Liberty assertion and process it */
            LibertyAssertionControl aCtl =
                new LibertyAssertionControl();
            AuthenticatorResult authnResult = 
                LibertyAuthenticator.authenticateArtifact(httpReq, artifact, aCtl);

            /* Evaluate the authentication result */
            if (authnResult == AuthenticatorResult.AUTHN_SUCCESS) {
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Successful";
                procResult.authnQueryResult.redirectedServer = aCtl.redirectedServer;
            } else if (authnResult == AuthenticatorResult.AUTHN_FAILED) {
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Failed";
                procResult.authnQueryResult.redirectedServer = aCtl.redirectedServer;
            } else {
                if (Config.debug) {
                    System.err.println("Error at LIBERTY assertion authentication");
                }
                procResult.failureCode = "samlp:Receiver";
            }
            return ANPRequestProcessorAction.RETURN_RESPONSE;
        } catch (Exception e) {
            if (Config.debug) {
                System.err.println(e);
            }
            return ANPRequestProcessorAction.FAILED;
        }
    }
}

