package ucb.eecs.sahara.sso.authn;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;
import javax.servlet.http.HttpServletRequest;
import ucb.eecs.sahara.sso.authn.IPFilterUtils;

import java.util.Timer;
import java.util.TimerTask;


/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class FirewallController {

    private static final String ADD = "Add";
    private static final String REMOVE = "Remove";

    public static boolean giveAccess(HttpServletRequest httpReq)
		  throws Exception
    {
        String ip = httpReq.getRemoteAddr();
	return changeIP_URLAccess(ip, null, ADD);
    }


    public static boolean giveURLAccess(HttpServletRequest httpReq,
				        String url)
		  throws Exception
    {
        String ip = httpReq.getRemoteAddr();
	return changeIP_URLAccess(ip, url, ADD);
    }


    public static boolean removeAccess(HttpServletRequest httpReq)
		  throws Exception
    {
        String ip = httpReq.getRemoteAddr();
	return changeIP_URLAccess(ip, null, REMOVE);
    }


    public static boolean removeURLAccess(HttpServletRequest httpReq,
				          String url)
		  throws Exception
    {
        String ip = httpReq.getRemoteAddr();
	return changeIP_URLAccess(ip, url, REMOVE);
    }


    public static boolean removeIP_URLAccess(String ip,
				             String url)
		  throws Exception
    {
	return changeIP_URLAccess(ip, url, REMOVE);
    }


    /* If no URL is specified, access to every URL is given/removed
     * Different options can be used as firewall. So far, just one:
     *     IP Filter is used if a manager for it is configured
     */
    private static boolean changeIP_URLAccess(String ip, String url,
					      String action)
		  throws Exception
    {
        if (Config.ipManager != null) {
            if (action.equals(ADD)) {
	        if (url == null) {
                    if (Config.debug) {
                        System.err.println("FirewallController: Adding address to IP filter");
                    }

                    if (!IPFilterUtils.addIP(ip)) {
                        throw new Exception ("FirewallController: Adding address to IP Filter failed");
                    }

                    /* Remove address after certain time interval */
                    //Create timer as daemon, so that it dies when the 
                    //invoking thread dies
                    Timer timer = new Timer(true);
                    IP_URLAccessTimeout task = new IP_URLAccessTimeout(ip, null);
                    timer.schedule(task, Config.accessTimeout * 1000);
	        } else {
                    if (Config.debug) {
                        System.err.println("FirewallController: Adding address and URL to IP filter");
                    }

                    if (!IPFilterUtils.addIP(url, ip)) {
                        throw new Exception ("FirewallController: Adding address and URL to IP Filter failed");
                    }

                    /* Remove address and URL after certain time interval */
                    //Create timer as daemon, so that it dies when the 
                    //invoking thread dies
                    Timer timer = new Timer(true);
                    IP_URLAccessTimeout task = new IP_URLAccessTimeout(ip, url);
                    timer.schedule(task, Config.urlAccessTimeout * 1000);
	        }
	    } else if (action.equals(REMOVE)) {
	        if (url == null) {
                    if (Config.debug) {
                        System.err.println("FirewallController: Removing address from IP filter");
                    }

                    if (!IPFilterUtils.removeIP(ip)) {
                        throw new Exception ("FirewallController: Removing address from IP Filter failed");
                    }
	        } else {
                    if (Config.debug) {
                        System.err.println("FirewallController: Removing address and URL from IP filter");
                    }

                    if (!IPFilterUtils.removeIP(url, ip)) {
                        throw new Exception ("FirewallController: Removing address and URL from IP Filter failed");
                    }
	        }
	    }

            DebugUtils.debugTime("FirewallController", "Firewall modified",
                                 DebugUtils.LOW_LEVEL_FUNC);
            return true;
        }
	else {
            if (Config.debug) {
                System.err.println(
		    "FirewallController: no available IP Filter Manager");
            }
            return false;
	}
    }
}


class IP_URLAccessTimeout extends TimerTask {
    String ip = null;
    String url = null;

    IP_URLAccessTimeout(String ip, String url) {
        this.ip = ip;
        this.url = url;
    }

    public void run () {
        try {
	    FirewallController.removeIP_URLAccess(ip, url);
        } catch (Exception e) {
            System.err.println("Error removing [" + ip + "," + url +
                               "] at timeout");
        }
    }
}


