package ucb.eecs.sahara.sso.authn;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;
import java.net.*;

/**
 * 
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class IPFilterUtils
{
    private static final int addIpPort = 9000;
    private static final int removeIpPort = 9001;
    private static final int addIpUrlPort = 9002;
    private static final int removeIpUrlPort = 9003;

    private static DatagramSocket sock = null;

    static {
        try {
	    sock = new DatagramSocket();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void finalize() throws Throwable {
        try {
            sock.close();
        } finally {
            super.finalize();
        }
    }


    public static boolean addIP (String ip)
    {
        DebugUtils.debugTime("IPFilterUtils", "Entering addIP(ip)",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            InetAddress ipAddress = InetAddress.getByName(ip);
            String ipName = ipAddress.getHostName();
            if (Config.debug) {
                System.err.println("IPFilterUtils: adding IP " + ipName);
            }

            String data = ipName;
	    return sendDatagram(data, addIpPort);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean removeIP (String ip)
    {
        DebugUtils.debugTime("IPFilterUtils", "Entering removeIP(ip)",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            InetAddress ipAddress = InetAddress.getByName(ip);
            String ipName = ipAddress.getHostName();
            if (Config.debug) {
                System.err.println("IPFilterUtils: removing IP " + ipName);
            }

            String data = ipName;
	    return sendDatagram(data, removeIpPort);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean addIP (String url, String ip)
    {
        DebugUtils.debugTime("IPFilterUtils", "Entering addIP(url, ip)",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            InetAddress ipAddress = InetAddress.getByName(ip);
            String ipName = ipAddress.getHostName();
            if (Config.debug) {
                System.err.println("IPFilterUtils: adding IP " + ipName +
				   " for URL " + url);
            }

            String data = ipName + "\n" + url + "\n";
	    return sendDatagram(data, addIpUrlPort);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean removeIP (String url, String ip)
    {
        DebugUtils.debugTime("IPFilterUtils", "Entering removeIP(url, ip)",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            InetAddress ipAddress = InetAddress.getByName(ip);
            String ipName = ipAddress.getHostName();
            if (Config.debug) {
                System.err.println("IPFilterUtils: removing IP " + ipName + " for URL " + url);
            }

            String data = ipName + "\n" + url + "\n";
	    return sendDatagram(data, removeIpUrlPort);
        } catch (Exception e)
        {
            e.printStackTrace();
            return false;
        }
    }

    private static boolean sendDatagram (String data, int port)
    {
        DebugUtils.debugTime("IPFilterUtils", "Entering sendDatagram",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            byte[] buff = data.getBytes();
	    DatagramPacket pack = new DatagramPacket(buff, buff.length,
				      InetAddress.getLocalHost(), port);
	    sock.send(pack);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }
}

