package ucb.eecs.sahara.sso.authn;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import com.sun.liberty.LibertyUserSession;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;


/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class SessionLocalAuthenticator {

    private static final String LOGIN_CONFIGURATION_ID = Config.loginConfID;


    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
        return lus;
    }


    public static void sessionCleanUp(HttpServletRequest httpReq) {
        HttpSession ses = httpReq.getSession(true);
        if (ses.getAttribute("liberty.session") != null) {
            ses.removeAttribute("liberty.session");
        }
    }


    public static AuthenticatorResult authenticate(HttpServletRequest httpReq,
                                                   final String userName,
                                                   final String userPassword)
		  throws Exception
    {
        Subject subj = null;
        if (!Config.debugAuthn) {
            if (Config.debug) { System.err.println("doing real authn"); }
            class MyCallbackHandler implements CallbackHandler {
                public void handle(Callback[] callbacks)
                    throws UnsupportedCallbackException {
                        for (int i = 0; i < callbacks.length; i++) {
                            // XXX: Should we support more callbacks here?  Maybe.
                            Callback c = callbacks[i];
                            if (c instanceof NameCallback) {
                                NameCallback nc = (NameCallback) c;
                                nc.setName(userName);
                            } else if (c instanceof PasswordCallback) {
                                PasswordCallback pc = (PasswordCallback) c;
                                pc.setPassword(userPassword.toCharArray());
                            } else {
                                throw new UnsupportedCallbackException
                                    (c, "unrecognized callback");
                            }
                        }
                    }
            }
            CallbackHandler myCallback = new MyCallbackHandler();
            try {
                LoginContext lctx = new LoginContext(LOGIN_CONFIGURATION_ID,
                        myCallback);
                lctx.login();
                subj = lctx.getSubject();
                if (Config.debug) {
		    System.err.println("real authn succeeds");
		}
                DebugUtils.debugTime("SessionLocalAuthenticator", "authn done",
                                     DebugUtils.LOW_LEVEL_FUNC);
            } catch (LoginException e) {
                if (Config.debug) { System.err.println("real authn failed"); }
            }
        } else {
            if (Config.debug) { System.err.println("doing fake authn"); }
            /* Hack! Log in any user where username == pass. */
            if (userName.equals(userPassword)) {
                subj = new Subject();
                java.security.Principal pr;
                pr = new com.sun.security.auth.SolarisPrincipal(userName);
                subj.getPrincipals().add(pr);
                if (Config.debug) { System.err.println("fake authn succeeds"); }
            }
	    else {
                if (Config.debug) { System.err.println("fake authn failed"); }
	    }
        }

        HttpSession ses = httpReq.getSession(true);
        if (subj == null) {
            /*
             * if auth failed, clears any previous identity since
             * it is no longer valid.
             * XXX: should we initiate a SingleLogout in this case?
             */
            ses.removeAttribute("liberty.session");
            if (Config.debug) { System.err.println("failed... removing session"); }
	    return AuthenticatorResult.AUTHN_FAILED;
        } else {
            try {
        	LibertyUserSession lus = getLibertySession(httpReq);
                lus.setUser(subj);
                if (lus.getUser() == null) {
                    throw new Exception("setUser(subj) failed");
                }
                if (Config.debug) {
                    System.err.println("succeeded... setUser");
                    System.err.println("lus: " + lus);
                }
                DebugUtils.debugTime("SessionLocalAuthenticator", "user set",
                                     DebugUtils.LOW_LEVEL_FUNC);
                return AuthenticatorResult.AUTHN_SUCCESS;
            } catch (Exception e) {
                if (Config.debug) { System.err.println("can't set user"); }
                ses.removeAttribute("liberty.session");
                throw new Exception("Cannot set user.", e);
            }
        }
    }
}
